/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package util;

import java.io.Console;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

/**
 *
 * @author andjse
 */
public class DirectoryIDUpdateUtility {

   private static final String USER_QUERY = "SELECT identity, web_user FROM fnd_user_tab";
   private static final String INSERT_STMT = "INSERT INTO directory_mapping_tbl(identity,samaccountname,userprincipalname) VALUES (?,?,?)";
   private final LdapEnvInfo ldapEnv = new LdapEnvInfo();
   private final DBEnvInfo dbEnv = new DBEnvInfo();

   private void readUserInput() {
      Console console = System.console();
      // LDAP
      ldapEnv.setProviderUrl(console.readLine("Enter URL to Active Directory: "));
      ldapEnv.setSearchBase(console.readLine("Enter Active Directory search base: "));
      ldapEnv.setUserName(console.readLine("Enter Active Directory username: "));
      ldapEnv.setPassWord(new String(console.readPassword("Enter Active Directory password: ")));
      ldapEnv.validate();
      // DB
      String prefix = "jdbc:oracle:thin:@";
      dbEnv.setConnectString(prefix+console.readLine("Enter Database connect string: "));
      dbEnv.setUserName(console.readLine("Enter Database username: "));
      dbEnv.setPassWord(new String(console.readPassword("Enter Database password: ")));
      dbEnv.validate();
   }

   private void loadTempTable() {
      // Connect to LDAP
      DirContext dirContext = connectToActiveDirectory();
      try {
         Class.forName("oracle.jdbc.OracleDriver");
      } catch (ClassNotFoundException e) {
         throw new RuntimeException("The Oracle JDBC-driver could not be loaded.", e);
      }
      // foreach match, insert into temp-table
      int usersCount = processUserData(dirContext);
      System.out.println(String.format("%d users processed.", usersCount));
   }

   private DirContext connectToActiveDirectory() {
      try {
         Hashtable<String, Object> env = new Hashtable<>();
         env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
         env.put(Context.REFERRAL, "follow");
         env.put(Context.SECURITY_AUTHENTICATION, "simple");
         env.put(Context.SECURITY_PRINCIPAL, this.ldapEnv.getUserName());
         env.put(Context.SECURITY_CREDENTIALS, this.ldapEnv.getPassWord());
         env.put("com.sun.jndi.ldap.connect.timeout", "10000");
         if (this.ldapEnv.getProviderUrl().toLowerCase().startsWith("ldaps://")) {
            env.put(Context.SECURITY_PROTOCOL, "ssl");
         }
         env.put(Context.PROVIDER_URL, this.ldapEnv.getProviderUrl());
         DirContext ctx = new InitialDirContext(env);
         return ctx;
      } catch (NamingException e) {
         throw new RuntimeException("A problem occurred when connecting to the Active Directory.", e);
      }
   }

   private int processUserData(DirContext ctx) {
      int usersCount = 0;
      String url = this.dbEnv.getConnectString();
      try (Connection c = DriverManager.getConnection(url, dbEnv.getUserName(), dbEnv.getPassWord())) {
         c.setAutoCommit(false);
         try (PreparedStatement query = c.prepareStatement(USER_QUERY); PreparedStatement insert = c.prepareStatement(INSERT_STMT)) {
            ResultSet rs = query.executeQuery();
            while (rs.next()) {
               String userID = rs.getString(1);
               String currentDirectoryID = rs.getString(2);
               String userPrincipalName = getUserPrincipalName(ctx, currentDirectoryID);
               if (!"".equals(userPrincipalName)) {
                  insert.setString(1, userID);
                  insert.setString(2, currentDirectoryID);
                  insert.setString(3, userPrincipalName.toUpperCase());
                  insert.execute();
                  usersCount++;
               }
            }
            rs.close();
         }
         c.commit();
      } catch (SQLException e) {
         throw new RuntimeException("An error ocurred when accessing the database.", e);
      }
      return usersCount;
   }

   private String getUserPrincipalName(DirContext ctx, String accountName) {
      try {
         String userPrincipalName = "";
         String searchFilter = "(&(objectClass=user)(sAMAccountName=" + accountName + "))";
         SearchControls searchControls = new SearchControls();
         searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE);
         NamingEnumeration<SearchResult> results = ctx.search(this.ldapEnv.getSearchBase(), searchFilter, searchControls);
         if (results.hasMoreElements()) {
            SearchResult searchResult = (SearchResult) results.nextElement();
            Attribute upn = searchResult.getAttributes().get("userPrincipalName");
            userPrincipalName = (String) upn.get();
         }
         return userPrincipalName;
      } catch (NamingException e) {
         throw new RuntimeException("An error occurred when searching Active Directory for user "+accountName, e);
      }
   }

   /**
    * @param args the command line arguments
    */
   public static void main(String[] args) {
      DirectoryIDUpdateUtility util = new DirectoryIDUpdateUtility();
      util.readUserInput();
      util.loadTempTable();
   }

}
